@@ -16,8 +16,8 @@ class Command(ScrapyCommand):
 
     def add_options(self, parser):
         ScrapyCommand.add_options(self, parser)
-        parser.add_option("--verbose", "-v", dest="verbose", action="store_true",
-                          help="also display twisted/python/platform info (useful for bug reports)")
+        parser.add_argument("--verbose", "-v", dest="verbose", action="store_true",
+                            help="also display twisted/python/platform info (useful for bug reports)")
 
     def run(self, args, opts):
         if opts.verbose:
