@@ -1,5 +1,6 @@
 import asyncio
 import sys
+from typing import Optional
 
 from scrapy import Spider
 from scrapy.crawler import CrawlerProcess
@@ -31,6 +32,7 @@ class UrlSpider(Spider):
 
 
 if __name__ == "__main__":
+    ASYNCIO_EVENT_LOOP: Optional[str]
     try:
         ASYNCIO_EVENT_LOOP = sys.argv[1]
     except IndexError:
