@@ -6,6 +6,7 @@ from twisted.internet.defer import Deferred
 from twisted.trial import unittest
 
 from scrapy import Spider, signals, Request
+from scrapy.utils.defer import maybe_deferred_to_future, deferred_to_future
 from scrapy.utils.test import get_crawler, get_from_asyncio_queue
 
 from tests.mockserver import MockServer
@@ -31,18 +32,38 @@ class DeferredPipeline:
 
 class AsyncDefPipeline:
     async def process_item(self, item, spider):
-        await defer.succeed(42)
+        d = Deferred()
+        from twisted.internet import reactor
+        reactor.callLater(0, d.callback, None)
+        await maybe_deferred_to_future(d)
         item['pipeline_passed'] = True
         return item
 
 
 class AsyncDefAsyncioPipeline:
     async def process_item(self, item, spider):
+        d = Deferred()
+        from twisted.internet import reactor
+        reactor.callLater(0, d.callback, None)
+        await deferred_to_future(d)
         await asyncio.sleep(0.2)
         item['pipeline_passed'] = await get_from_asyncio_queue(True)
         return item
 
 
+class AsyncDefNotAsyncioPipeline:
+    async def process_item(self, item, spider):
+        d1 = Deferred()
+        from twisted.internet import reactor
+        reactor.callLater(0, d1.callback, None)
+        await d1
+        d2 = Deferred()
+        reactor.callLater(0, d2.callback, None)
+        await maybe_deferred_to_future(d2)
+        item['pipeline_passed'] = True
+        return item
+
+
 class ItemSpider(Spider):
     name = 'itemspider'
 
@@ -99,3 +120,10 @@ class PipelineTestCase(unittest.TestCase):
         crawler = self._create_crawler(AsyncDefAsyncioPipeline)
         yield crawler.crawl(mockserver=self.mockserver)
         self.assertEqual(len(self.items), 1)
+
+    @mark.only_not_asyncio()
+    @defer.inlineCallbacks
+    def test_asyncdef_not_asyncio_pipeline(self):
+        crawler = self._create_crawler(AsyncDefNotAsyncioPipeline)
+        yield crawler.crawl(mockserver=self.mockserver)
+        self.assertEqual(len(self.items), 1)
